<%@ tag body-content="scriptless" dynamic-attributes="args" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@ taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn" %>
<%@ taglib tagdir="/WEB-INF/tags" prefix="tags" %>
<%@ attribute name="name" required="true" type="java.lang.String" %>
<%@ attribute name="label" %>
<%@ attribute name="items" type="java.lang.Object" description="Collection, iterator, map or array to loop over." %>
<%@ attribute name="begin" type="java.lang.Integer" %>
<%@ attribute name="end" type="java.lang.Integer" %>
<%@ attribute name="step" type="java.lang.Integer" %>
<%@ attribute name="selected" type="java.lang.Object" description="Use comma separator for multiple selected values" %>
<%@ attribute name="required" type="java.lang.Boolean" %>
<%@ attribute name="disabled" type="java.lang.Boolean" %>
<%@ attribute name="template" fragment="true" %>
<%@ variable name-given="labelTag" scope="NESTED" %>
<%@ variable name-given="selectTag" scope="NESTED" %>

<c:if test="${empty items and (not empty begin and not empty end and begin < end)}">
    <jsp:useBean id="items" class="java.util.LinkedHashMap" scope="page"/>
    <c:forEach var="itemValue" begin="${begin}" end="${end}" step="${step > 0 ? step : 1}">
        <c:set target="${items}" property="${itemValue}" value="${itemValue}" />
    </c:forEach>
</c:if>
<c:if test="${disabled}">
    <c:set target="${args}" property="disabled" value="disabled" />
</c:if>
<tags:tagDynamicAttributes defaultIdValue="${name}" dynamicAttributes="${args}"/>

<c:set var="labelTag">
    <c:if test="${not empty label}">
        <tags:label htmlfor="${tagIdValue}" required="${required}" label="${label}"/>
    </c:if>
</c:set>

<c:set var="isMultipleSelection" value="${args['multiple'] != null}" />


<c:set var="selectTag">
    <select name="${name}" id="${tagIdValue}" ${tagAttributes}>
        <c:forEach var="item" items="${items}">
            <c:set var="isMapEntry" value="${item.class.memberClass and fn:endsWith(item.class.canonicalName, 'Map.Entry')}" />
            <c:set var="key"><c:out value="${isMapEntry ? item.key : item}"/></c:set>
            <c:set var="value"><c:out value="${isMapEntry ? item.value : item}"/></c:set>

            <c:set var="isSelectedItem" value="${false}"/>
            <c:choose>
                <c:when test="${isMultipleSelection}">
                    <c:forTokens var="token" items="${selected}" delims=",">
                        <c:if test="${not isSelectedItem and (fn:trim(token) eq key or fn:trim(token) eq value)}">
                            <c:set var="isSelectedItem" value="true"/>
                        </c:if>
                    </c:forTokens>
                </c:when>
                <c:otherwise>
                    <c:set var="isSelectedItem" value="${selected eq key or selected eq value}" />
                </c:otherwise>
            </c:choose>

            <c:choose>
                <c:when test="${isSelectedItem}"><option value="${key}" selected="selected">${value}</option></c:when>
                <c:otherwise><option value="${key}">${value}</option></c:otherwise>
            </c:choose>
        </c:forEach>
    </select>
</c:set>

<c:choose>
    <c:when test="${not empty template}">
        <jsp:invoke fragment="template"/>
    </c:when>
    <c:otherwise>
        ${labelTag} ${selectTag}
    </c:otherwise>
</c:choose>