function List(list) {
    this._constructor(list);
}
(function() {

    List.PCS_SELECTED_INDEX = "selectedIndex";

    List.prototype = {

        list: null,
        listeners: null,

        _constructor: function(list) {
            this.listeners = [];
            this.list = list;
            this.list.onchange = function() {
                this.firePropertyChange(List.PCS_SELECTED_INDEX, null, this.getSelectedIndex());
            }.bind(this);
        },

        addPropertyChangeListener: function(listener) {
            if (listener && Object.isFunction(listener.propertyChange)) {
                this.listeners.push(listener);
            }
        },

        removePropertyChangeListener: function(listener) {
            for (var i = 0; i < this.listeners.length; ++i) {
                if (this.listeners[i] == listener) {
                    delete this.listeners[i];
                    break;
                }
            }
        },

        firePropertyChange: function(propertyName, oldValue, newValue) {
            if (oldValue != null && newValue != null && oldValue === newValue) {
                return;
            }

            var event = {propertyName: propertyName, oldValue: oldValue, newValue: newValue};

            for (var i = 0; i < this.listeners.length; ++i) {
                this.listeners[i].propertyChange(event);
            }
        },

        addItem: function(text, value) {
            var item = document.createElement("option");
            item.appendChild(document.createTextNode(text));
            item.value = value || text;
            item.title = value || text;
            this.list.appendChild(item);
        },

        removeItem: function(value) {
            var index = this.indexOf(value);

            if (index == -1) {
                return;
            }

            var selectedIndex = this.getSelectedIndex();
            var item = this.list.options[index];
            this.list.removeChild(item);

            if (selectedIndex == index) {
                var count = this.getItemCount();
                this.setSelectedIndex(index < count ? index : count - 1);
            }
        },

        getItems: function() {
            var result = new Array();
            for (var i = 0; i < this.list.options.length; ++i) {
                result.push(this.list.options[i].value);
            }
            return result;
        },

        getItem: function(index) {
            if (!this._isValidIndex(index)) {
                return null;
            }
            return this._asItem(this.list.options[index]);
        },

        getItemCount: function() {
            return this.list.options.length;
        },

        indexOf: function(value) {
            for (var i = 0; i < this.list.options.length; ++i) {
                if (this.list.options[i].value == value) {
                    return i;
                }
            }
            return -1;
        },

        getSelectedIndex: function() {
            return this.list.selectedIndex;
        },

        setSelectedIndex: function(index) {
            if (index > this.getItemCount()) {
                return;
            }

            try {
                var oldSelectedIndex = this.getSelectedIndex();
                this.list.selectedIndex = index;
                this.firePropertyChange(List.PCS_SELECTED_INDEX, oldSelectedIndex, index);
            } catch(e) {
                // fails in IE6
                (function(self) {
                    self.list.selectedIndex = index;
                    self.firePropertyChange(List.PCS_SELECTED_INDEX, oldSelectedIndex, index);
                }.delay(10, this));
            }
        },

        getSelectedItems: function() {
            var result = new Array();

            for (var i = 0; i < this.list.options.length; ++i) {
                var option = this.list.options[i];
                if (option.selected) {
                    result.push(this._asItem(option));
                }
            }

            return result;
        },

        _asItem: function(option) {
            return {text: option.text, value: option.value, toString: function(){return this.text;}};
        },

        _isValidIndex: function(index) {
            return !(isNaN(index) || index < 0 || index > this.getItemCount() - 1);
        }
    };
})();