var ImportKeyDialog = Class.create(ModalDialog, {

    form: null,
    channel: null,
    listener: null,

    _constructor: function(url) {
        this.channel = new IFrameChannel("ImportKeyDataChannel");
        this.channel.setPreventCaching(true);
        this.channel.open(url);
        this.channel.addLoadListener(this._handleFormLoaded.bind(this));

        this.getContent().appendChild(this.channel.asElement());
        Utils.setStyle(this.channel.asElement(), {width: 0, height: 0, display: "none"});

        this.setTitle(ImportKeyDialog.TITLE);
        this.addButton(ModalDialog.BUTTON_OK, ImportKeyDialog.Buttons.OK, this.onSubmit.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, ImportKeyDialog.Buttons.CANCEL, this.onCancel.bind(this));
    },

    getId: function() {
        return "ImportKeyDialog";
    },

    getForm: function() {
        return this.form;
    },

    getDoneMessage: function() {
        return this._getMetaValue("x-dialog-reason") || "";
    },

    show: function(listener) {
        this.listener = Object.extend(ImportKeyDialog.Listener, listener);
        this.form = null;

        this._updateSubmitButton(ImportKeyDialog.Messages.LOADING, false);
        Utils.setStyle(this.channel.asElement(), {width: "100%", height: "100%", display: "block"});
        this.setSize(520, 240);
        this.setVisible(true);
        this.channel.send("");
    },

    hide: function() {
        this.listener = null;
        this.form = null;
        this.channel.close();
        this.dispose();
    },

    onSubmit: function() {
        this._updateSubmitButton(ImportKeyDialog.Messages.IMPORTING, false);
        this.listener.onSubmit(this);
    },

    onCancel: function() {
        this.listener.onCancel(this);
    },

    _handleFormLoaded: function() {
        if (this._isDone()) {
            this.listener.onDone(this);
        } else {
            this._initDialogForm();
            this._updateSubmitButton(ImportKeyDialog.Buttons.OK, true);
        }
    },

    _initDialogForm: function() {
        var forms = this.channel.asDocument().forms;
        if (forms.length != 0) {
            this.form = forms[0];
            Event.observe(this.form, "submit", this.onSubmit.bind(this));
        } else {
            this.form = null;
        }
    },

    _isDone: function() {
        return (this._getMetaValue("x-dialog-control") || "").toLocaleLowerCase() == "done";
    },

    _updateSubmitButton: function(text, enabled) {
        this.setButtonEnabled(ModalDialog.BUTTON_OK, enabled);
        var button = this.getButton(ModalDialog.BUTTON_OK);
        button.value = text;
    },

    _getMetaValue: function(httpEquiv) {
        var tags = this.channel.asDocument().getElementsByTagName("meta");
        for (var i = 0; i < tags.length; ++i) {
            var meta = tags[i];
            if (meta.httpEquiv == httpEquiv) {
                return meta.content;
            }
        }
        return null;
    }

});

ImportKeyDialog.TITLE = "Import Key";
ImportKeyDialog.Buttons = {OK: 'OK', CANCEL: 'Cancel'};
ImportKeyDialog.Messages = {LOADING: "Loading...", IMPORTING: "Importing..."};
ImportKeyDialog.Listener = {
    onSubmit: function(dialog) {},
    onCancel: function(dialog) {},
    onDone: function(dialog) {}
};