var ScriptChannel = Class.create({

    id: null,
    url: null,
    scripts: null,
    preventCaching: false,

    _constructor: function(id, preventCaching) {
        this.id = id || String(new Date().getTime());
        this.preventCaching = !!preventCaching;
        this.scripts = [];
    },

    open: function(url) {
        this.url = url;
    },

    send: function(data, callback) {
        var script = this._createScript(this._createUri(data));
        this._bindEvents(script, callback);

        var holder = this._scriptHolder();
        this.scripts.push(holder.insertBefore(script, holder.firstChild));
    },

    close: function() {
        while (this.scripts.length != 0) {
            this._cleanup(this.scripts.shift());
        }
    },

    _cleanup: function(script) {
        this._unbindEvents(script);

        if (script && script.parentNode) {
            script.parentNode.removeChild(script);
        }
    },

    _remove: function(script) {
        var index = this.scripts.indexOf(script);
        if (index != -1) {
            this.scripts[index] = null;
            this.scripts.splice(index, 1);
        }
    },

    _handleScriptLoad: function(script, callback) {
        var scriptLoaded = !script.readyState || /loaded|complete/.test(script.readyState);
        if (scriptLoaded) {
            this._cleanup(script);
            this._remove(script);

            if (Object.isFunction(callback)) {
                callback();
            }
        }
    },

    _bindEvents: function(script, callback) {
        script.onload = script.onreadystatechange = this._handleScriptLoad.bind(this, script, callback);
    },

    _unbindEvents: function(script) {
        if (Object.isElement(script)) {
            script.onload = null;
            script.onreadystatechange = null;
        }
    },

    _scriptHolder: function() {
        return document.head || document.getElementsByTagName("HEAD")[0] || document.documentElement;
    },

    _createScript: function(url) {
        var result = document.createElement("script");
        result.language = "javascript";
        result.type = "text/javascript";
        result.async = "async";
        result.src = url;
        return result;
    },

    _createUri: function(data) {
        var result = this.url + (data || "");
        if (this.preventCaching) {
            result += (result.indexOf("?") == -1 ? "?" : "&");
            result += "nocache=" + new Date().getTime();
        }
        return encodeURI(result);
    }
});