function StandardFileChooser(listener) {
    this._constructor(listener);
}
(function() {

    StandardFileChooser.Css = {DEFAULT: "filechooser", HIDDEN: "hidden"};
    StandardFileChooser.Input = {NAME: "filename", TITLE: ""};

    var STATIC_FILE_INPUT_ID = "filelistiemac";

    StandardFileChooser.prototype = {

        listener: null,
        chooser: null,
        selectedChoosers: null,
        allowDuplicate: null,
        staticFileInput: null,

        _constructor: function(listener) {
            this.listener = listener;
            this.selectedChoosers = [];
            this.allowDuplicate = true;
            this.dynamicFileInput = !Utils.Browser.IEMac;
        },

        setDuplicateAllowed: function(allowed) {
            this.allowDuplicate = allowed;
        },

        install: function(form) {
            if (!this.dynamicFileInput) {
                return;
            }

            var staticFileInput = $(STATIC_FILE_INPUT_ID);
            if (staticFileInput) {
                staticFileInput.parentNode.removeChild(staticFileInput);
            }

            this.chooser = form.appendChild(this._createChooser());

            var oldHandler = form.onsubmit || function(){return true;};
            form.onsubmit = function() {
                this.onBeforeSubmit();
                return oldHandler();
            }.bind(this);
        },

        destroy: function() {
            this._removeChooser(this.chooser);

            while(this.selectedChoosers.length > 0) {
                this._removeChooser(this.selectedChoosers.pop());
            }
        },

        onBeforeSubmit: function() {
            this._removeChooser(this.chooser);

            while(this.selectedChoosers.length > 0) {
                this.selectedChoosers.pop().disabled = false;
            }
        },

        remove: function(filename) {
            return this.removeAll([filename]);
        },

        removeAll: function(files) {
            var modified = false;

            for (var i = 0; i < this.selectedChoosers.length; ++i) {
                var chooser = this.selectedChoosers[i];
                if (files.indexOf(chooser.value) != -1) {
                    chooser.parentNode.removeChild(chooser);
                    this.selectedChoosers.splice(i, 1);
                    modified = true;
                }
            }

            return modified;
        },

        toString: function() {
            return "StandardFileChooser";
        },

        _handleFileSelected: function() {
            var value = this.chooser.value;
            if (!this.allowDuplicate && this._isDuplicate(value)) {
                this.chooser.value = null;
                return;
            }

            var selectedChooser = this.chooser;
            this.selectedChoosers.push(selectedChooser);
            this.chooser = selectedChooser.form.appendChild(this._createChooser());
            Utils.addCssClass(selectedChooser, StandardFileChooser.Css.HIDDEN);
            selectedChooser.disabled = true;

            // IE and Safari returns full path to file
            var separator = (value.indexOf('/') != -1) ? '/' : '\\';
            var filename = value.substr(value.lastIndexOf(separator) + 1);

            this.listener.onFileSelect([filename]);
        },

        _createChooser: function() {
            var chooser = Utils.createFileInput(StandardFileChooser.Input.NAME);
            chooser.name += this.selectedChoosers.length;
            chooser.title = StandardFileChooser.Input.TITLE;
            chooser.size = 1;
            chooser.tabIndex = -1;
            Utils.addCssClass(chooser, StandardFileChooser.Css.DEFAULT);

            chooser.onchange = this._handleFileSelected.bind(this);
            chooser.onkeypress = function() { this.blur(); };

            return chooser;
        },

        _removeChooser: function(chooser) {
            if (chooser) {
                chooser.onchange = null;
                chooser.onkeypress = null;
                chooser.parentNode.removeChild(chooser);
            }
        },

        _getChooserFor: function(filename) {
            for (var i = 0; i < this.selectedChoosers.length; ++i) {
                var chooser = this.selectedChoosers[i];
                if (chooser.value == filename) {
                    return chooser;
                }
            }
            return null;
        },

        _isDuplicate: function(value) {
            return this._getChooserFor(value) != null;
        }
    };
})();