var UploadForm = Class.create({

    url: null,
    uploader: null,
    formSelectionDialog: null,
    uploadFormDialog: null,
    channel: null,
    forms: null,
    selectedFormName: null,
    error: null,

    _constructor: function(uploader, url) {
        this.uploader = uploader;
        this.url = url || UploadForm.URL;
        this.channel = new IFrameChannel("UploadFormChannel");
        this.channel.setPreventCaching(true);
        this.forms = new Array();
    },

    init: function() {
        if (this.formSelectionDialog == null) {
            this.formSelectionDialog = new UploadFormDialog(this.forms, {
                onSubmit: this._handleFinishFormSelection.bind(this),
                onCancel: this._handleCancelFormSelection.bind(this),
                onCancelUpload: this._handleCancelUpload.bind(this)
            });
        }
        if (this.uploadFormDialog == null) {
            this.uploadFormDialog = new ModalDialog("UploadFormDialog");
            this.uploadFormDialog.addButton(ModalDialog.BUTTON_OK, UploadFormDialog.Buttons.OK, this._handleFormSubmit.bind(this));
            this.uploadFormDialog.addButton(ModalDialog.BUTTON_CANCEL, UploadFormDialog.Buttons.CANCEL, this._handleFormCancel.bind(this));
        }
    },

    addForm: function(name, description) {
        this.forms.push({name: name, description: description});
    },

    onStart: function() {
        this.error = null;

        if (this.forms.length != 0) {
            this.init();
            this._displayFormSelectionDialog();
        } else {
            this.onSuccess();
        }
    },

    onSuccess: function() {
        if (this.uploadFormDialog != null) {
            this.uploadFormDialog.setVisible(false);
        }
        this.uploader.upload();
    },

    onError: function(message) {
        this.error = message;
    },

    onFormLoaded: function() {
        Utils.setStyle(this.channel.asElement(), {width: "100%", height: "100%", display: "block"});
        var form = this._getFormNamed(this.selectedFormName);

        if (form != null) {
            form.onsubmit = this.onFormSubmit.bind(this, form);
            this.uploadFormDialog.setButtonEnabled(ModalDialog.BUTTON_OK, true);
            //this.uploadFormDialog.pack();
        }
    },

    onFormSubmit: function(form) {
        if (form == null) {
            return false;
        }

        if (!this._isValidForm(form)) {
            alert(UploadForm.Messages.INVALID_REQUIRED);
            return false;
        }

        form.submit();
    },

    _displayFormSelectionDialog: function() {
        var viewport = document.viewport.getDimensions();
        this.formSelectionDialog.setSize(viewport.width * 0.4, viewport.height * 0.37);
        this.formSelectionDialog.setTitle(UploadForm.Messages.TITLE);
        this.formSelectionDialog.setVisible(true);
    },

    _displayUploadFormDialog: function() {
        this.channel.open(this.url);
        this.uploadFormDialog.setContent(this.channel.asElement());
        var viewport = document.viewport.getDimensions();
        this.uploadFormDialog.setSize(viewport.width * 0.5, viewport.height * 0.6);
        this.uploadFormDialog.setButtonEnabled(ModalDialog.BUTTON_OK, false);
        this.uploadFormDialog.setVisible(true);

        this.channel.send({form: this.selectedFormName, filename: this.uploader.getFilenames()});
    },

    _handleFinishFormSelection: function() {
        this.selectedFormName = this.formSelectionDialog.getSelectedItem();
        this.formSelectionDialog.setVisible(false);

        this._displayUploadFormDialog();
    },

    _handleCancelFormSelection: function() {
        this.formSelectionDialog.setVisible(false);
        this.uploader.upload();
    },

    _handleCancelUpload: function() {
        this.formSelectionDialog.setVisible(false);
        this.uploader.cancel();
    },

    _handleFormSubmit: function() {
        if (this.error != null) {
            alert(this.error);
            return;
        }

        var form = this._getFormNamed(this.selectedFormName);
        this.onFormSubmit(form);
    },

    _handleFormCancel: function() {
        this.uploadFormDialog.setVisible(false);
        this._displayFormSelectionDialog();
    },

    _getFormNamed: function(name) {
        var forms = this.channel.asDocument().forms;
        for (var i = 0; i < forms.length; ++i) {
            var candidate = forms[i];
            if (candidate.name == name) {
                return candidate;
            }
        }
        return null;
    },

    _isValidForm: function(form) {
        for (var i = 0; i < form.elements.length; ++i) {
            var input = form.elements[i];
            var requiredNode = input.getAttributeNode(UploadForm.REQUIRED_FIELD_ATTRIBUTE);
            var isRequiredField = Tag.INPUT_TYPE_HIDDEN != input.type && (requiredNode && requiredNode.specified);

            if (!isRequiredField) {
                continue;
            }

            if (Utils.isRadioInput(input) && form.elements[input.name].length > 1) {
                var radioGroup = form.elements[input.name];
                if (!this._hasSelectedItem(radioGroup)) {
                    return false;
                }
            } else {
                var value = $F(input) || "";
                if (value.trim().length == 0) {
                    return false;
                }
            }
        }
        return true;
    },

    _hasSelectedItem: function(items) {
        for (var i = 0; i < items.length; ++i) {
            if (items[i].checked) {
                return true;
            }
        }
        return false;
    }
});

UploadForm.URL = "";
UploadForm.REQUIRED_FIELD_ATTRIBUTE = "required";
UploadForm.Messages = {TITLE: "Form Data", INVALID_REQUIRED: "Please provide all required fields."};
