var UploadFormDialog = Class.create(ModalDialog, {

    listener: null,
    formsBox: null,
    formDescElement: null,

    _constructor: function(forms, listener) {
        this.listener = listener;

        this.formsBox = document.createElement(Tag.SELECT);
        this.formsBox.name = "formname";
        this.formsBox.id = this.formsBox.name;
        this.formsBox.onchange = this._update.bind(this);

        this.formDescElement = document.createElement(Tag.SPAN);
        this.formDescElement.id = "formdescription";

        this.setContent(this._createContent(forms));

        this.addButton(ModalDialog.BUTTON_OK, UploadFormDialog.Buttons.OK, this.onSubmit.bind(this));
        this.addButton(ModalDialog.BUTTON_CANCEL, UploadFormDialog.Buttons.CANCEL, listener.onCancel.bind(listener));
        this.addButton(ModalDialog.BUTTON_CANCEL, UploadFormDialog.Buttons.CANCEL_UPLOAD, this.onCancel.bind(this));

        var viewport = document.viewport.getDimensions();
        this.setSize(viewport.width * 0.4, viewport.height * 0.37);
    },

    getId: function() {
        return "UploadFormSelectionDialog";
    },

    getSelectedItem: function() {
        return $F(this.formsBox);
    },

    setVisible: function(visible) {
        this._super.setVisible.call(this, visible);
        this._update();
    },

    setSize: function(width, height) {
        width = Math.max(UploadFormDialog.Size.MINIMUM.width, width);
        height = Math.max(UploadFormDialog.Size.MINIMUM.height, height);
        this._super.setSize.call(this, width, height);
    },

    onSubmit: function() {
        this.listener.onSubmit();
    },

    onCancel: function() {
        this.listener.onCancelUpload();
    },

    _createContent: function(forms) {
        for (var i = 0; i < forms.length; ++i) {
            var form = forms[i];

            var item = document.createElement(Tag.OPTION);
            item.appendChild(document.createTextNode(form.name));
            item.value = form.name;
            item.title = form.description;
            this.formsBox.appendChild(item);
        }

        var panel = document.createElement(Tag.DIV);

        var infoPanel = document.createElement(Tag.DIV);
        infoPanel.appendChild(document.createTextNode(UploadFormDialog.Messages.INFO));
        Utils.setStyle(infoPanel, "padding: 2em 1.5em 2em 4em;");
        panel.appendChild(infoPanel);

        var formPanel = document.createElement(Tag.DIV);
        formPanel.appendChild(Utils.createLabel(UploadFormDialog.Labels.FORM, this.formsBox.id));
        formPanel.appendChild(this.formsBox);
        formPanel.appendChild(document.createElement("BR"));
        formPanel.appendChild(Utils.createLabel(UploadFormDialog.Labels.DESCRIPTION, this.formDescElement.id));
        formPanel.appendChild(this.formDescElement);
        Utils.setStyle(formPanel, "padding-left: 5em; line-height: 2");
        panel.appendChild(formPanel);

        return panel;
    },

    _update: function() {
        if (Object.isElement(this.formsBox)) {
            var description = this._getFormDescripton($F(this.formsBox));
            this._setDescription(description || "");
        }
    },

    _getFormDescripton: function(name) {
        for (var i = 0; i < this.formsBox.options.length; ++i) {
            var item = this.formsBox.options[i];
            if (name == item.value) {
                return item.title;
            }
        }
        return null;
    },

    _setDescription: function(description) {
        var descElement = document.createTextNode(description);

        if (this.formDescElement.firstChild) {
            this.formDescElement.replaceChild(descElement, this.formDescElement.firstChild);
        } else {
            this.formDescElement.appendChild(descElement);
        }
    }

});

UploadFormDialog.Buttons = {OK: "OK", CANCEL: "Cancel", CANCEL_UPLOAD: "Cancel Upload"};
UploadFormDialog.Labels = {FORM: "Form", DESCRIPTION: "Description"};
UploadFormDialog.Messages = {INFO: "Select form to use for file upload and click OK or click Cancel to upload files without form data."};
UploadFormDialog.Size = {MINIMUM: {width: 400, height: 245}};